﻿using SQLite;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Thick.Blowing.Server.Model
{
    /// <summary>
    /// 订单记录
    /// </summary>
    [Table("Profile")]
    public class DB_Profile
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 产品名称
        /// </summary>
        [PropertyIndex(1)]
        public string PName { get; set; }

        /// <summary>
        /// 订单号
        /// </summary>
        [PropertyIndex(2)]
        public string OrderNo { get; set; }

        /// <summary>
        /// 卷号
        /// </summary>
        [PropertyIndex(3)]
        public string Number { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        [PropertyIndex(4)]
        public DateTime StartTime { get; set; }

        /// <summary>
        /// 结束时间
        /// </summary>
        [PropertyIndex(5)]
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 已经结束
        /// </summary>
        [PropertyIndex(6)]
        public bool IsFinished { get; set; }

        /// <summary>
        /// 目标值
        /// </summary>
        [PropertyIndex(7)]
        public double Target { get; set; }

        /// <summary>
        /// 工艺%
        /// </summary>
        [PropertyIndex(8)]
        public double TolerancePercent { get; set; }

    }


    /// <summary>
    /// 厚度记录
    /// </summary>
    [Table("ScanData")]
    public class DB_ScanData
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 订单记录ID
        /// </summary>
        [PropertyIndex(1)]
        public Int64 ProfileID { get; set; }

        /// <summary>
        /// 测量数据开始时间
        /// </summary>
        [PropertyIndex(2)]
        public DateTime Time { get; set; }

        /// <summary>
        /// 测量数据结束时间
        /// </summary>
        [PropertyIndex(3)]
        public DateTime EndTime { get; set; }

        /// <summary>
        /// 旋转方向 是反向
        /// </summary>
        [PropertyIndex(4)]
        public bool IsBackw { get; set; }

        /// <summary>
        /// 旋转1周的时间 min
        /// </summary>
        [PropertyIndex(5)]
        public double RPeriod { get; set; }

        /// <summary>
        /// 旋转次数
        /// </summary>
        [PropertyIndex(6)]
        public int RCnt { get; set; }

        /// <summary>
        /// 复位区号
        /// </summary>
        [PropertyIndex(7)]
        public int OrgBoltNo { get; set; }

        /// <summary>
        /// 旋转角度 °
        /// </summary>
        [PropertyIndex(8)]
        public double RAngle { get; set; }

        /// <summary>
        /// 膜距离 m
        /// </summary>
        [PropertyIndex(9)]
        public double FilmLength { get; set; }

        /// <summary>
        /// 线速度
        /// </summary>
        [PropertyIndex(10)]
        public double FilmVelocity { get; set; }

        /// <summary>
        /// 斜率补偿
        /// </summary>
        [PropertyIndex(11)]
        public double K { get; set; }

        /// <summary>
        /// 1幅数据
        /// </summary>
        [PropertyIndex(12)]
        public string Thicks { get; set; }

        /// <summary>
        /// 分区表
        /// </summary>
        [PropertyIndex(13)]
        public string Boltmap { get; set; }
    }

    /// <summary>
    /// 异常记录
    /// </summary>
    [Table("Error")]
    public class DB_Error
    {
        [Key]
        [PropertyIndex(0)]
        public Int64 ID { get; set; }

        /// <summary>
        /// 发生的时间
        /// </summary>
        [PropertyIndex(1)]
        public DateTime Time { get; set; }

        /// <summary>
        /// 异常代码
        /// </summary>
        [PropertyIndex(2)]
        public int ErrCode { get; set; }

        /// <summary>
        /// true=异常是发生了,false=异常关闭
        /// </summary>
        [PropertyIndex(3)]
        public bool IsOn { get; set; }

        /// <summary>
        /// 异常描述
        /// </summary>
        [PropertyIndex(4)]
        public string Descrption { get; set; }

    }
}
