﻿
using FLY.Thick.Blowing.IService;
using FLY.Thick.Blowing.IService.IShareDBServicePack;
using FObjBase;
using SQLite;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Thick.Blowing.Server.Model
{
    public class HistoryDB : IShareDBService
    {
        DBModel dBModel;
        public LocalDB localDB;

        #region IDBShareService
        public event ProfileChangedEventHandler ProfileChanged;
        public event ScanDataAddedEventHandler ScanDataAdded;
        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dBModel"></param>
        /// <param name="localDB"></param>
        public void Init(DBModel dBModel, LocalDB localDB)
        {
            this.dBModel = dBModel;
            this.localDB = localDB;
        }


        /// <summary>
        /// 按时间删除数据库
        /// </summary>
        /// <param name="month"></param>
        public void KeepDBSize(int month)
        {
            if (month <= 3)
                month = 3;

            DateTime del_time = DateTime.Now - TimeSpan.FromDays(month * 30);

            List<string> sqls = new List<string>();
            sqls.Add(
                $"DELETE FROM {dBModel.TbProfile.TableName}" +
                $" WHERE EndTime < {del_time.ToStringOfSQLiteFieldType()}");

            sqls.Add(
                $"DELETE FROM {dBModel.TbScanData.TableName}" +
                $" WHERE Time < {del_time.ToStringOfSQLiteFieldType()}");

            sqls.Add(
                $"DELETE FROM {dBModel.TbError.TableName}" +
                $" WHERE Time < {del_time.ToStringOfSQLiteFieldType()}");

            dBModel.sqliteHelper.QueryTran(sqls);

        }

        /// <summary>
        /// 保存产品参数
        /// </summary>
        public void AddProfile(
            DB_Profile profile
            )
        {
            //把上一次结束掉
            if ((localDB.CurrProfile != null) && localDB.IsProfileFinished == false)
            {
                FinishProfile();
            }

            //添加 数据库必要的 field
            localDB.ProfileStartTime = DateTime.Now;
            localDB.ProfileEndTime = DateTime.Now;
            localDB.IsProfileFinished = false;

            profile.ID = dBModel.TbProfile.FreeID;
            profile.StartTime = localDB.ProfileStartTime;
            profile.EndTime = localDB.ProfileEndTime;
            

            //SQL
            List<string> sqls = new List<string>();
            sqls.Add(SQLiteHelper.GetInsertCommandText(profile));

            dBModel.sqliteHelper.QueryTranAsync(sqls);


            //本地数据备份
            localDB.CurrProfile = profile;

            #region IDBShareService
            ProfileChanged?.Invoke(this, new ProfileChangedEventArgs()
            {
                isInsert = true,
                profile = profile
            });
            #endregion
        }

        /// <summary>
        /// 修改产品参数
        /// </summary>
        /// <param name="profile"></param>
        public void UpdateProfile(
                DB_Profile profile
            )
        {
            //添加 数据库必要的 field
            profile.ID = localDB.CurrProfile.ID;
            profile.StartTime = localDB.ProfileStartTime;
            profile.EndTime = localDB.ProfileEndTime;
            
            //SQL
            List<string> sqls = new List<string>();
            sqls.Add(SQLiteHelper.GetUpdateCommandText(profile, $"WHERE ID = {profile.ID}"));

            dBModel.sqliteHelper.QueryTranAsync(sqls);


            //本地数据备份
            localDB.CurrProfile = profile;

            #region IDBShareService
            ProfileChanged?.Invoke(this, new ProfileChangedEventArgs()
            {
                profile = profile
            });
            #endregion
        }

        /// <summary>
        /// 下辊，结束当前产品生产
        /// </summary>
        public void FinishProfile()
        {

            if (localDB.CurrProfile == null)
                return;

            //SQL
            dBModel.sqliteHelper.ExecuteNonQuery(
                $"UPDATE {dBModel.TbProfile.TableName} SET IsFinished = {true} WHERE ID = {localDB.CurrProfile.ID}");

            localDB.IsProfileFinished = true;

            localDB.CurrProfile.IsFinished = true;

            #region IDBShareService
            ProfileChanged?.Invoke(this, new ProfileChangedEventArgs()
            {
                profile = localDB.CurrProfile
            });
            #endregion
        }

        /// <summary>
        /// 不需设置 DB_ScanData 内 ProfileID, 自动设置！
        /// </summary>
        public void AddScanData(
            LC_ScanData scanData)
        {

            //添加 数据库必要的 field
            scanData.ID = dBModel.TbScanData.FreeID;
            scanData.ProfileID = localDB.CurrProfile.ID;

            localDB.ProfileEndTime = scanData.Time;

            //SQLs
            List<string> sqls = new List<string>();
            var db_scandata = AutoMapper.Mapper.Map<DB_ScanData>(scanData);
            sqls.Add(SQLiteHelper.GetInsertCommandText(db_scandata));

            string s_endtime = localDB.ProfileEndTime.ToStringOfSQLiteFieldType();

            sqls.Add($"UPDATE {dBModel.TbProfile.TableName} SET EndTime={s_endtime} WHERE ID = {localDB.CurrProfile.ID}");
            dBModel.sqliteHelper.QueryTranAsync(sqls);

            //本地数据备份
            localDB.CurrProfile.EndTime = localDB.ProfileEndTime;


            #region IDBShareService
            ScanDataAdded?.Invoke(this, new ScanDataAddedEventArgs()
            {
                scandata = scanData
            });
            #endregion
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="db_errorData"></param>
        public void AddErrorData(
            DB_Error db_errorData
            )
        {
            //添加 数据库必要的 field
            db_errorData.ID = dBModel.TbError.FreeID;
            
            //SQLs
            List<string> sqls = new List<string>();
            sqls.Add(SQLiteHelper.GetInsertCommandText(db_errorData));
            
            dBModel.sqliteHelper.QueryTranAsync(sqls);
        }

        #region IDBShareService
        public void GetProfile(AsyncCBHandler asyncDelegate, object asyncContext)
        {
            if (localDB.CurrProfile != null)
            {
                asyncDelegate.Invoke(asyncContext, localDB.CurrProfile);
            }
            else
            {
                asyncDelegate.Invoke(asyncContext, null);
            }
        }
        #endregion
    }
}
