﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;
using System.Net;
using System.Diagnostics;
using System.ComponentModel;

using Misc;
using FObjBase;
using FlyADBase;
using FLY.Thick.Base.Common;
using FLY.Thick.Base.IService;
using FLY.Thick.Base.Server;
using FLY.Thick.RemoteHistory;
using FLY.Thick.BulkDataModule;
using FLY.Thick.Blowing.Server.Model;

namespace FLY.Thick.Blowing.Server
{
    public class TDGage : ITDGageService
    {
        #region 延时推送 MARKNO
        const int MARKNO_UPDATEV = 1;
        const int MARKNO_INITHISTORY = 2;
        const int MARKNO_PUSH_ACCESS = 3;
        #endregion
        #region 参数--------------------------------------------------------------------
        #region 初始化------------------------------------------------------------------
        /// <summary>
        /// 参数目录 m_path;数据文件的 根目录默认是 "Gage1", 不能有 @"\"
        /// 目前也是对外的服务器目录
        /// </summary>
        public string mParamDirectory;
        /// <summary>
        /// 名字 如:gage0
        /// </summary>
        public string mName;
        #endregion

        #region 数据--------------------------------------------------------------------

        /// <summary>
        /// 当前基本简单参数
        /// </summary>
        public DynArea mDynArea;


        /// <summary>
        /// 运行参数
        /// </summary>
        public BlowingFixProfile mProfile;

        /// <summary>
        /// AD曲线
        /// </summary>
        public CurveCollection mCurve;

        /// <summary>
        /// 报警系统
        /// </summary>
        public FLY.OBJComponents.Server.WarningSystem mWarning;

        /// <summary>
        /// 密码系统，只提供给客户端，在服务器没用。。。
        /// </summary>
        public Password mPassword;

        #region 运行模式

        /// <summary>
        /// AD卡
        /// </summary>
        public FlyADBase.FlyAD7 mFlyAD;

        /// <summary>
        /// GM管理器
        /// </summary>
        GageModeManager mGMManager;

        /// <summary>
        /// 定点模式
        /// </summary>
        public GM_Fix mGMFix;
        /// <summary>
        /// 吹膜测厚模式.定点
        /// </summary>
        public GM_BlowingFix mGMRenZiJiaFix;

        #endregion

        /// <summary>
        /// 系统参数
        /// </summary>
        public SysParam mSysParam;


        #region BULKDATA
        /// <summary>
        /// N幅历史数据缓存
        /// </summary>
        public BulkDataService mBulk;
        public BulkDataServiceClientAdv mBulkClient;

        DBModel mDBModel;
        public HistoryDB mHistoryDB;
        LocalDB mLocalDB;
        public BulkDB mBulkDB;

        #endregion

        #endregion

        #endregion

        public TDGage(string nam)
        {
            mName = nam;
            mParamDirectory = mName;// @"D:\" + m_name;

            if (!System.IO.Directory.Exists(mParamDirectory))
            {
                System.IO.Directory.CreateDirectory(mParamDirectory);
            }
            System.Environment.CurrentDirectory = mParamDirectory;

            #region 提取保存的数据
            #region 数据备份。。。。

            BackupFile.Add("sysparam.xml", BackupFile.xml_isFileOK);

            BackupFile.Add("curve.xml", BackupFile.xml_isFileOK);

            BackupFile.Add("renzijiafix.xml", BackupFile.xml_isFileOK);

            BackupFile.Add("blowingdetect.xml", BackupFile.xml_isFileOK);

            BackupFile.CheckResult cr = BackupFile.Check();

            #endregion

            mSysParam = new SysParam(null);//系统参数
            
            mProfile = new BlowingFixProfile(null);//运行配置参数

            mCurve = new CurveCollection(null);//AD转thick

            mPassword = new Password(null);

            mDBModel = new DBModel();
            mDBModel.Init();

            mLocalDB = new LocalDB();

            mHistoryDB = new HistoryDB();
            mHistoryDB.Init(mDBModel, mLocalDB);
            mHistoryDB.KeepDBSize(6);

            mBulkDB = new BulkDB();
            mBulkDB.Init(mDBModel, mHistoryDB, mLocalDB);

            #endregion


            #region 动态数据
            mDynArea = new DynArea();
            mDynArea.Version = Version;
            mDynArea.GageType = GAGE_TYPE.BlowingFix;
            Misc.BindingOperations.SetBinding(mProfile.Param, "PName", mDynArea, "ProductName");
            Misc.BindingOperations.SetBinding(mProfile.Param, "Target", mDynArea, "Target");
            Misc.BindingOperations.SetBinding(mProfile.Param, "Alarm", mDynArea, "Alarm");
            Misc.BindingOperations.SetBinding(mProfile.Param, "Comp", mDynArea, "Comp");

            mDynArea.PropertyChanged += new PropertyChangedEventHandler(mDynArea_PropertyChanged);


            //---------------------------------------------------------------------------------------------------------------
            //GageModeManager_Create
            mGMManager = new GageModeManager();
            Misc.BindingOperations.SetBinding(mGMManager, "State", mDynArea, "ControllerState");

            //---------------------------------------------------------------------------------------------------------------
            //FlyAD7_Create
            mFlyAD = new FlyAD7();
            
            Misc.BindingOperations.SetBinding(mSysParam, "FLYAD7_HasCRC",mFlyAD, "HasCRC", BindingOperations.BindingMode.TwoWay);
            
            mFlyAD.Connect(mSysParam.FLYAD7EP);
            Misc.BindingOperations.SetBinding(mFlyAD, "LocalEP", mSysParam, "FLYAD7EP");

            Misc.BindingOperations.SetBinding(mFlyAD, "IStatus", mDynArea, "IStatus");
            Misc.BindingOperations.SetBinding(mFlyAD, "OStatus", mDynArea, "OStatus");
            Misc.BindingOperations.SetBinding(mFlyAD, "Position", mDynArea, "Position");
            Misc.BindingOperations.SetBinding(mFlyAD, "Position2", mDynArea, "Position2");
            Misc.BindingOperations.SetBinding(mFlyAD, "Surplus", mDynArea, "Hrs");
            Misc.BindingOperations.SetBinding(mFlyAD, "IsConnected", mDynArea, "FLYADIsConnect");


            //---------------------------------------------------------------------------------------------------------------
            //Warning_Create
            mWarning = new OBJComponents.Server.WarningSystem();
            mWarning.RingEvent += () =>
            {
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(4 - 1),
                    (UInt16)~Misc.MyBase.BIT(4 - 1));
            };
            mWarning.ResetEvent += () => {
                mFlyAD.SetOutput(
                    (UInt16)Misc.MyBase.BIT(4 - 1),
                    (UInt16)Misc.MyBase.BIT(4 - 1));
            };
            

            //---------------------------------------------------------------------------------------------------------------
            //GM_Fix_Create
            mGMFix = new GM_Fix(mFlyAD);
            mGMFix.Init(mDynArea, AD2Thick);
            mGMManager.AddGM(mGMFix);




            //---------------------------------------------------------------------------------------------------------------
            //GM_RenZiJiaFix_Create
            mGMRenZiJiaFix = new GM_BlowingFix(mFlyAD);

            mGMManager.AddGM(mGMRenZiJiaFix);

            //---------------------------------------------------------------------------------------------------------------
            //Bulkdate_Create
            mBulk = new BulkDataService(mGMRenZiJiaFix.NBolts, 1, 400);
            mBulkClient = new BulkDataServiceClientAdv(mBulk);

            
            mGMRenZiJiaFix.Init(mBulk, new AD2ThickHandler(AD2Thick), mDynArea, mProfile.Param, mHistoryDB, mBulkDB );

            mGMRenZiJiaFix.EPCSampled += new GM_BlowingFix.EPCSampledEventHandler(mGMRenZiJiaFix_EPCSampled);

            mGMRenZiJiaFix.Start();


            ErrNoForCheckInit();

            #endregion
        }

        void mGMRenZiJiaFix_EPCSampled(int ad)
        {
            CurveCollection pCurve = mCurve;

            mDynArea.SampleAD[0] = ad;
            List<CurveCollection.ExChange> list = new List<CurveCollection.ExChange>();
            if (pCurve.Curves.Count == 0)
                return;


            list.Add(new CurveCollection.ExChange() { OrgAD = pCurve.Curves[0].AD, CurrAD = ad });
            pCurve.ModifyExChange(list);
            pCurve.ReviseCurve();
        }
        #region 异常检测
        class ErrNoForCheckItem
        {
            public FLY.OBJComponents.Common.ERRNO errno;
            public Func<bool> conditional;
            /// <summary>
            /// 持续时间，单位s
            /// </summary>
            public int cnt;

            int timer = -1;
            public FLY.OBJComponents.Server.WarningSystem mWarning;
            public void Clear()
            {
                if (timer == 0)
                {
                    mWarning.Add(
                        errno.Code,
                        errno.Descrption,
                        FLY.OBJComponents.Common.ERR_STATE.OFF);
                }
                timer = -1;
            }
            public void OnPoll()
            {
                if (conditional())
                {
                    if (timer < 0)
                        timer = cnt;
                    else
                    {
                        if (timer > 0)
                            timer--;

                        if (timer == 0)
                        {
                            mWarning.Add(
                                errno.Code,
                                errno.Descrption,
                                FLY.OBJComponents.Common.ERR_STATE.ON);
                        }
                    }
                }
                else
                {
                    Clear();
                }
            }
        }

        List<ErrNoForCheckItem> ErrNoForCheckItems = new List<ErrNoForCheckItem>();
        void ErrNoForCheckInit()
        {

            ErrNoForCheckItems.Add(//AD值太小
                new ErrNoForCheckItem()
                {
                    errno = FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_AD_MIN,
                    cnt = 20,
                    conditional = delegate()
                    {
                        return mDynArea.AD < (mDynArea.ADMax * 0.01);
                    }
                });

            ErrNoForCheckItems.Add(//AD值太大
                new ErrNoForCheckItem()
                {
                    errno = FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_AD_MAX,
                    cnt = 20,
                    conditional = delegate()
                    {
                        return (mDynArea.AD > (mDynArea.ADMax * 0.98));
                    }
                });

            foreach (ErrNoForCheckItem item in ErrNoForCheckItems)
            {
                item.mWarning = mWarning;
            }

            FObjBase.PollModule.Current.Poll_Config(
                FObjBase.PollModule.POLL_CONFIG.ADD,
                onpoll_checkErrNo, TimeSpan.FromSeconds(1));
        }



        bool isInit_onpoll_checkErrNo;

        void onpoll_checkErrNo() //1秒执行一次
        {
            if (!isInit_onpoll_checkErrNo)
            {
                isInit_onpoll_checkErrNo = true;
                checkDynAreaError();
            }

            if (!mDynArea.FLYADIsConnect)
            {
                foreach (ErrNoForCheckItem item in ErrNoForCheckItems)
                    item.Clear();
            }
            else
            {
                foreach (ErrNoForCheckItem item in ErrNoForCheckItems)
                    item.OnPoll();
            }
        }
        #endregion
        void mDynArea_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Hrs")
            {
                if (mDynArea.Hrs < 48 && mDynArea.Hrs > 0)//授权限制提醒
                {

                    mWarning.Add(
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE_TIP.Code,
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE_TIP.Descrption,
                        OBJComponents.Common.ERR_STATE.ON);
                }
            }
            else if (e.PropertyName == "SecuteLock") //授权限制
            {
                if (mDynArea.SecuteLock)
                {
                    mWarning.Add(
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE.Code,
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE.Descrption,
                        OBJComponents.Common.ERR_STATE.ON);
                }
                else
                {
                    mWarning.Add(
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE.Code,
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_LICENSE.Descrption,
                        OBJComponents.Common.ERR_STATE.ON);
                }
            }
            else if (e.PropertyName == "FLYADIsConnect")//连接断开
            {
                if (!mDynArea.FLYADIsConnect)
                {
                    mWarning.Add(
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Code,
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Descrption,
                        OBJComponents.Common.ERR_STATE.ON);

                }
                else
                {
                    mWarning.Add(
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Code,
                        FLY.Thick.Blowing.Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Descrption,
                        OBJComponents.Common.ERR_STATE.OFF);
                }
            }

        }

        void checkDynAreaError()
        {
            if (mDynArea.Hrs < 48 && mDynArea.Hrs > 0)//授权限制提醒
            {
                mWarning.Add(
                    Common.ERRNOs.BASE_ERRNO_LICENSE_TIP.Code,
                    Common.ERRNOs.BASE_ERRNO_LICENSE_TIP.Descrption,
                    OBJComponents.Common.ERR_STATE.ON);
            }
            if (mDynArea.SecuteLock)
            {
                mWarning.Add(
                    Common.ERRNOs.BASE_ERRNO_LICENSE.Code,
                    Common.ERRNOs.BASE_ERRNO_LICENSE.Descrption,
                    OBJComponents.Common.ERR_STATE.ON);
            }
            if (!mDynArea.FLYADIsConnect)
            {
                mWarning.Add(
                    Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Code,
                    Common.ERRNOs.BASE_ERRNO_FLYAD7DISCONNECTED.Descrption,
                    OBJComponents.Common.ERR_STATE.ON);
            }

        }

        #region AD卡密码操作，主机版本号操作--------------------------------------------------------
        public string Version//获取版本号
        {
            get
            {
                System.Reflection.Assembly ma = System.Reflection.Assembly.GetEntryAssembly();
                FileInfo fi = new FileInfo(ma.Location);
                FileVersionInfo mfv = FileVersionInfo.GetVersionInfo(ma.Location);
                return mfv.FileVersion;
            }

        }
        #endregion
        #region 推送处理
        private int AD2Thick(int ad)
        {
            int thick = Misc.MyBase.NULL_VALUE;

            if (Misc.MyBase.ISVALIDATA(ad))
            {
                thick = mCurve.AD2Value(ad, AD2ValueFlag.Revised);
                
                thick = (int)(thick * mProfile.Param.Comp);
            }
            return thick;
        }






        #endregion

        #region ITDGageService

        //扫描
        public void SetDeviceState(bool devicestate)
        {
            //if (mInitParam.DataValidSrc == DATAVALIDSRC.BYPLC)
            //{
            //    mDynArea.DataValid = devicestate;
            //}
        }
        public void StartP1()
        {
            //if (mGMScan != null)
            //    mGMScan.Start();
            //else if (mGMRenZiJiaScan != null)
            //    mGMRenZiJiaScan.Start();
        }

        public void StartP2(STARTP2_MODE mode)
        {
            //StartP2(mode, 0);
        }
        public void StartP2(STARTP2_MODE mode, int targetpos)
        {
            switch (mode)
            {
                case STARTP2_MODE.ORG:
                    //mGMOrg.Start();
                    break;
                case STARTP2_MODE.STOP:

                    //mGMStop.Start();
                    break;
                case STARTP2_MODE.FORW:
                case STARTP2_MODE.FORW_UNTIL_STOP:
                    //mGMForw.Start();
                    break;
                case STARTP2_MODE.BACKW:
                case STARTP2_MODE.BACKW_UNTIL_STOP:
                    //mGMBackw.Start();
                    break;
                case STARTP2_MODE.RUNTO:
                    //mGMRunto.Start(targetpos);
                    break;
            }
        }

        public void RollDown()
        {
            mDynArea.RollDown++;
        }

        #endregion
    }
}
